/*
 * Copyright (c) 2003-2004 Jakub Jermar
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup arm32mm	
 * @{
 */
/** @file
 */

#include <arch/mm/page.h>
#include <genarch/mm/page_pt.h>
#include <mm/page.h>
#include <align.h>
#include <config.h>
#include "../aux_print/printf.h"

void page_arch_init(void)
{
  	aux_printf("page_arch_init\n");

    uintptr_t cur;
	int flags;

	page_mapping_operations = &pt_mapping_operations;

	flags = PAGE_CACHEABLE;
    const unsigned maxmem = ALIGN_DOWN(config.memory_size, FRAME_SIZE);
    
    for (cur = 0; cur < maxmem; cur += FRAME_SIZE) {
		page_mapping_insert(AS_KERNEL, PA2KA(cur), cur, flags);
	}    
	//TODO set page fault routines
	// no problem no ... kernel doesn't do page faults
	
    
	asm volatile (
    "ldr r0, =0x55555555       \n"
    "mrc p15, 0, r0, c3, c0, 0 \n" //set domain acces rights to client <==> take rights from page tables
    "mcr p15, 0, r0, c1, c0, 0 \n" // get current setting of system ... register 1 isn't only for memmory management
    "ldr r1, =0xFFFFFE8D       \n" // mask to disable aligment checks, System, Rom bit disable 
    "and r0, r0, r1            \n"
    "ldr r1, =0x00000001       \n" // mask to enable paging
    "orr r0, r0, r1            \n"
    "mrc p15, 0, r0, c1, c0, 0 \n" // store setting
    :
    :
    : "r0", "r1"
    );
}

/** Map device into kernel space. */
uintptr_t hw_map(uintptr_t physaddr, size_t size)
{
	/* TODO */
	return NULL;
}

/** @}
 */

